@@ -36,8 +36,8 @@ class AutoCrosswalk(object):
         self.unique_to = []
         
         # Instantiate matchers
-        self.numericmatcher = NumericMatcher()
-        self.textmatcher = TextMatcher()
+        self.numericmatcher = NumericMatcher(remove_special_characters=True)
+        self.textmatcher = TextMatcher(use_lower_characters=True)
         self.contextmatcher = ContextMatcher(normalize=True,
                                              verbose=self.verbose,
                                              model_name_or_path="all-roberta-large-v1")
@@ -277,8 +277,7 @@ class AutoCrosswalk(object):
             
                 for k in key_from:
                     P_temp[P_temp.index.get_level_values(key) == k] = self.numericmatcher.compute_similarity(a=k,
-                                                                                                             b=key_to,
-                                                                                                             remove_special_characters=True)
+                                                                                                             b=key_to)
                  
                 # Sanity check
                 if ((P_temp>1).any() | (P_temp<0).any()).any():
@@ -312,8 +311,7 @@ class AutoCrosswalk(object):
             
                 for k in key_from:
                     P_temp[P_temp.index.get_level_values(key) == k] = self.textmatcher.compute_similarity(a=k,
-                                                                                                          b=key_to,
-                                                                                                          use_lower_characters=True)
+                                                                                                          b=key_to)
                  
                 # Sanity check
                 if ((P_temp>1).any() | (P_temp<0).any()).any():
@@ -366,6 +364,9 @@ class AutoCrosswalk(object):
                 key_from = P.index.get_level_values(level=key).tolist()
                 key_to = P.columns.get_level_values(level=key).tolist()
                 
+                
+                
+                
                 # Get similarities                
                 similarities = self.contextmatcher.compute_similarity(a=key_from, b=key_to, metric="cosine")
 
@@ -636,7 +637,7 @@ class AutoCrosswalk(object):
         
         if use_existing_transition_matrix:                        
             # If we are using an existing transition matrix, we need to make sure that all unique keys are present
-            # If not all keys are present, we overwrite exisiting transition matrix
+            # If not all keys are present, we extend exisiting transition matrix
             
             # Get current unique from and to
             unique_from = transition_matrix_null.index
@@ -655,7 +656,7 @@ class AutoCrosswalk(object):
                 
                 if self.verbose:
                     bg.exceptions.print_warning("Some keys are missing in existing transition matrix. \nRe-estimating transition matrix and overwriting existing")
-                
+                                                                
                 # Some keys are missing. Hence, we estimate transition matrices again
                 self.transition_matrix = self._estimate_transition_matrix(numeric_key=numeric_key,
                                                                           text_key=text_key,
@@ -671,7 +672,7 @@ class AutoCrosswalk(object):
             transition_matrix = self.transition_matrix
                 
         else:
-            # Estimate new probability matrix
+            # Estimate new probability matrix (also used to enforce self-completeness)
             transition_matrix = self._estimate_transition_matrix(numeric_key=numeric_key,
                                                                  text_key=text_key,
                                                                  context_key=context_key,
@@ -702,8 +703,7 @@ class AutoCrosswalk(object):
         
         all_keys = numeric_key+text_key+context_key
         self._check_cols(df=df_from,cols=all_keys)
-        self._check_cols(df=df_to,cols=all_keys)
-        
+        self._check_cols(df=df_to,cols=all_keys)        
         self._check_prefix(prefix=prefix, df=df_from)
         
         # Check weights
@@ -736,7 +736,7 @@ class AutoCrosswalk(object):
         # ---------------------------------------------------------------------
         # Estimate transition proabability matrix
         # ---------------------------------------------------------------------
-        self.transition_matrix = self.estimate_transition_matrix(df_from=df_from,
+        transition_matrix = self.estimate_transition_matrix(df_from=df_from,
                                                             df_to=df_to,
                                                             use_existing_transition_matrix=use_existing_transition_matrix,
                                                             numeric_key=numeric_key,
@@ -757,7 +757,7 @@ class AutoCrosswalk(object):
                 
         if self.prioritize_exact_match:
             # Find exact matches
-            crosswalk_exact = self._find_exact_match(transition_matrix=self.transition_matrix,
+            crosswalk_exact = self._find_exact_match(transition_matrix=transition_matrix,
                                                      which_priority=which_priority)
             
             # Update existing crosswalk
@@ -768,19 +768,31 @@ class AutoCrosswalk(object):
         ## Find nbest matches
         # Mask those not found by now
         mask_find = crosswalk.isna().any(axis=1)
-        mask_lookup = self.transition_matrix["average"].index.isin(crosswalk.loc[mask_find].index)
+        mask_lookup = transition_matrix["average"].index.isin(crosswalk.loc[mask_find].index)
 
         if mask_lookup.sum()>0:
             # Find nbest candidates
-            crosswalk_nbest = self._find_nbest_match(transition_matrix=self.transition_matrix["average"].loc[mask_lookup],
+            crosswalk_nbest = self._find_nbest_match(transition_matrix=transition_matrix["average"].loc[mask_lookup],
                                                      n=self.n_best_match)
     
             crosswalks.append(crosswalk_nbest)
 
         if self.enforce_completeness:
             
-            # Transpose all transition matrices
-            transition_matrix_to = self._transpose_transition_matrix(transition_matrix=self.transition_matrix)
+            # # NB! We comment this out, because it is not enought to transpose matrix. We need to re-estimate!
+            # # Transpose all transition matrices
+            # transition_matrix_to = self._transpose_transition_matrix(transition_matrix=self.transition_matrix)
+            
+            # Estimate new transition matrix
+            transition_matrix_to = self.estimate_transition_matrix(df_from=df_to,
+                                                                   df_to=df_from,
+                                                                   use_existing_transition_matrix=False,
+                                                                   numeric_key=numeric_key,
+                                                                   text_key=text_key,
+                                                                   context_key=context_key,
+                                                                   weights=weights)
+            
+            
             
             # Make sure that all keys in "df_to" are also matched
             crosswalk_to = self._prepare_crosswalk(df=df_to,
@@ -907,11 +919,11 @@ class AutoCrosswalk(object):
 
             # Generate crosswalk
             self.crosswalk = self.generate_crosswalk(df_from=df_from,
-                                                   df_to=df,
-                                                    numeric_key=numeric_key,
-                                                    text_key=text_key,
-                                                    context_key=context_key,
-                                                    weights=weights)
+                                                     df_to=df,
+                                                     numeric_key=numeric_key,
+                                                     text_key=text_key,
+                                                     context_key=context_key,
+                                                     weights=weights)
             
             # Perform crosswalk
             df_updated = self.perform_crosswalk(crosswalk=self.crosswalk,
